#pragma once

#include <vector>
#include <iostream>
#include <string>
#include <cstring>
#include <optional>

#ifdef __linux__
#include <netdb.h>
#include <arpa/inet.h>
#elif WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock2.h>
#include <ws2ipdef.h>
#include <ws2tcpip.h>
#endif



namespace Sockets
{
    /// Internet Protocol Version
    /// @see https://www.geeksforgeeks.org/differences-between-ipv4-and-ipv6/
    enum class InternetProtocol : uint8_t
    {
        IPv4, IPv6
    };

    struct Hostname
    {
        InternetProtocol ProtocolVersion;
        std::string Host;
    };

    struct DNSResult
    {
        bool Success;
        std::vector<Hostname> ResolvedHostnames;
    };

    /// Resolve DNS for a given hostname.
    /// This function resolves the DNS for a given hostname and returns found IPaddress strings contained in a DNSResult struct.
    static DNSResult ResolveDNS(const std::string& hostname)
    {
        DNSResult result;
        result.ResolvedHostnames = std::vector<Hostname>();

        std::vector<std::string> resolved_hostnames;

        const char *cstr_hostname = hostname.c_str();
        struct addrinfo hints, *res, *p;
        int status;
        char ipstr[INET6_ADDRSTRLEN];

        std::memset(&hints, 0, sizeof hints);
        hints.ai_family = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;

        if ((status = getaddrinfo(cstr_hostname, NULL, &hints, &res)) != 0)
        {
            std::cerr << "getaddrinfo: " << gai_strerror(status) << std::endl;
            result.Success = false;
            return result;
        }

        std::cout << "IP addreses for " << hostname << ":" << std::endl;

        for (p = res; p != NULL; p = p->ai_next)
        {
            void* addr;
            std::string ipver;

            Hostname hostdata;


            if (p->ai_family == AF_INET) // IPv4
            {
                struct sockaddr_in *ipv4 = (struct sockaddr_in *)p->ai_addr;
                addr = &(ipv4->sin_addr);

                hostdata.ProtocolVersion = InternetProtocol::IPv4;

            } else {
                struct sockaddr_in6 *ipv6 = (struct sockaddr_in6 *)p->ai_addr;
                addr = &(ipv6->sin6_addr);

                hostdata.ProtocolVersion = InternetProtocol::IPv6;
            }

            // Convert the IP to a string and print it
            inet_ntop(p->ai_family, addr, ipstr, sizeof ipstr);

            hostdata.Host = ipstr;

            result.ResolvedHostnames.push_back(hostdata);
        }

        freeaddrinfo(res); // free the linked list
        return result;
    }
}
