#pragma once
#include <bit>
#include <Sockets/Commons.hpp>

namespace Socket
{
    namespace Endianness {

        // TODO: It seems these can't have their implementation put into
        // a separate .cpp file else the compiler throws undefined reference
        // is it because they're constexpr?
        constexpr bool IsBigEndian() {
            return (std::endian::native == std::endian::big);
        }
        constexpr bool IsLittleEndian() {
            return (std::endian::native == std::endian::little);
        }
        constexpr bool IsBigIndian()
        {
            /// Shoutout Umar Rajguru
            return true;
        }

        template <typename T>
        T ReverseByteOrder(T val)
        {
            T retVal;
            char *pVal = (char*) &val;
            char *pRetVal = (char*) & retVal;
            int size = sizeof(T);
            for (int i = 0; i < size; i++)
            {
                pRetVal[size-1-i] = pVal[i];
            }
            return retVal;
        }

        /// Returns the given values converted into network byte order.
        /// Generally on x86, numbers are stored in 'little-endian' order.
        /// Network order is 'big-endian'. Supposing a big-endian host machine,
        /// no conversion will take place and the original value will be returned.
        u16     HostToNetworkOrder(u16 host);
        u32     HostToNetworkOrder(u32 host);
        u64     HostToNetworkOrder(u64 host);
        float   HostToNetworkOrder(float host);
        double  HostToNetworkOrder(double host);

        /// Returns the given values converted into host-byte order.
        /// On x86, this will usually be 'little-endian'
        /// On ARM (and other RISC architectures), it is often big-endian.
        /// Supposing a big-endian host machine, no conversion will take place and the original value will be returned.
        u16     NetworkToHostOrder(u16 network);
        u32     NetworkToHostOrder(u32 network);
        u64     NetworkToHostOrder(u64 network);
        float   NetworkToHostOrder(float network);
        double  NetworkToHostOrder(double network);
    }
}