#pragma once

#include <iostream>
#include <string>
#ifdef __linux__
#include <netdb.h>
#elif WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <winsock2.h>
#endif
#include <memory>

namespace Socket {

	class SocketException : public std::exception {
	protected:
		const char * message;
	public:
        explicit SocketException(const std::string& msg) : message(msg.c_str()) {}
		SocketException(const char *msg) : message(msg) {}
		const char * what() const noexcept override {
			return message;
		}
	};

    class BadUriException : public SocketException {
    public:
        BadUriException(const char *msg) : SocketException{msg} {}
    };

	class SocketBindingException : public SocketException {
	public:
		SocketBindingException(char *msg) : SocketException{msg} {}
        SocketBindingException(const std::string& msg) : SocketException{msg.c_str()} {}
	};

	class SocketConnectException : public SocketException {
	public:
		SocketConnectException(char *msg) : SocketException{msg} {}
        SocketConnectException(const std::string& msg) : SocketException{msg.c_str()} {}
	};

	class SocketCreationException : public SocketException {
	public:
		SocketCreationException(char * msg) : SocketException{msg} {}
        SocketCreationException(const std::string& msg) : SocketException{msg.c_str()} {}
	};

	class SocketListenException : public SocketException {
	public:
		SocketListenException(char * msg) : SocketException{msg} {}
	};

	class SocketAcceptException : public SocketException {
	public:
		SocketAcceptException(char * msg) : SocketException{msg} {}
	    SocketAcceptException(const std::string& msg) : SocketException{msg.c_str()} {}
    };

	class SocketSendException : public SocketException {
	public:
        SocketSendException(const std::string& msg) : SocketException{msg.c_str()} {}
		SocketSendException(char * msg) : SocketException{msg} {}
	};

	class SocketReceiveException : public SocketException {
	public:
        SocketReceiveException(const std::string& msg) : SocketException{msg.c_str()} {}
		SocketReceiveException(char * msg) : SocketException{msg} {}
	};

	class SocketCloseException : public SocketException {
	public:
		SocketCloseException(char * msg) : SocketException{msg} {}
        SocketCloseException(const std::string& msg) : SocketException{msg.c_str()} {}
	};
}
