/* UDP Reliability Spec
 * Chase Carlson, Josh O'Leary
 *
 * All packet modes include an 8-bit checksum hash for verification, as standard. If this checksum hash fails validation either:
 * If Reliable: reply NACK
 * If Unreliable: drop the packet and do not pass onto application layer.
 *
 * Packet hash may be replaced with just encrypting the packet in-transit and if the encrypted data is corrupted/modified, the packet will fail to decrypt, not being passed on.
 *
 * Packet Reliability Modes
 *
 * Unreliable - No re-transmission or sequencing is attempted. Packet is dropped if
 * UnreliableOrdered - Only accepts packets with a higher-than-last-recieved sequence number.
 * Reliable - Re-transmission when ack is not recieved, or NACK (no-ACK) is recieved. All packets must be ACKed, but may appear in arbitrary order.
 * ReliableOrdered - UDP with retransmission. Packet sequence must be higher than last packet sequence. Does NOT require old packets to be retransmitted.  For example, if you get 1,2, and 5, you ACK 5 and the sender knows NOT to retransmit 3 and 4 because they are no longer needed.
 * ReliableSequenced - UDP with same retransmission protocol as reliable ordered, however, sequence numbers must be SEQUENTIAL, 1->2, 2->3, no 2->4. Send a NACK for every packet sequence ID which was NOT included in the range, i.e. if 2 was received and then 6 was received, then if 4 was received, you must not pass the packets onto the application layer until 3 and 5 are retransmitted. This is most similar to TCP
 *
 * Additional Considerations
 * Retransmission Limit
 * Packet Time-to-retransmit
 * Packet Time-to-die
 * UDP blocking vs Nonblocking (Chase uses nonblocking)
 * MTU size & reassembly
 *
 * Terms
 * NACK (No-ACK) - Lets the sender know a packet was not accepted. Often given when a packet contains an invalid checksum hash.
 * CheckSum Hash
 * Sequence Number - The number of a given packet, in a sequence.
 */

#pragma once


namespace Sockets
{
    namespace UdpReliability
    {

    }
}