#pragma once

#include <Sockets/IPAddress.hpp>
#include <string>
#if defined(__linux__) || defined(__APPLE__)

#elif defined(_WIN32)
#include <ws2tcpip.h>
#endif



// Game-specific Network code written against GameNetworkingSocket API

// GameNetworkingServer
// GameNetworkingClient
// GameNetworkingSocket
// UdpReliability
// TcpServer
// TcpClient
// UdpServer
// UdpClient
// UdpSocket_Old
// TcpSocket_Old
// Berkeley Sockets API

#ifndef INPORT_ANY
#define INPORT_ANY 0
#endif

namespace Socket
{

    // TODO: Implement Blocking and Nonblocking UDP

    using u8 = uint8_t;

	typedef struct sockaddr_in sockaddr_in_t;
	typedef struct sockaddr sockaddr_t;
	typedef std::vector<uint8_t> msg_t;

    // UdpSocket_Old is a class wrapper around BSD Sockets API for the UDP protocol.
	class UdpSocket_Old {
	public:

        /// Default Constructor which creates an empty, uninitialized, and unconnected UDP socket.
		UdpSocket_Old() {}
		UdpSocket_Old(int family, int flags);
		UdpSocket_Old(int socket, addrinfo info, bool connected, bool bound);
        ~UdpSocket_Old() {}
	
		UdpSocket_Old(const UdpSocket_Old &socket) = default;
		UdpSocket_Old &operator=(const UdpSocket_Old &socket) = delete;
		void Open();
		int Close();
		bool IsClosed() const { return sock < 0;}
		void Bind(const IPAddress& ipAddress);
		void Bind(uint16_t portno);
		void BindAny();
        void BindAny(uint16_t& portno);

        bool Connected() const;

        /// In UDP, the client does not form a connection with the server like in TCP, and instead, it just sends a datagram.
        /// Similarly, the server need not accept a connection and just waits for datagrams to arrive. There is no 3 way handshake.
        /// It just checks for any immediate errors and stores the peer's IP address and port number.
        void Connect(const std::string& hostname, u16 port);
        void Connect(const IPAddress& ipaddr);
        void Connect(u16 port);

		[[nodiscard]] IPAddress GetSelfIP() const {return self_addr;}
		[[nodiscard]] IPAddress GetPeerIP() const {return peer_addr;}


        /// Sends a message to the bound peer only.
        int Send(const u8 *data, unsigned int length, int flags = 0);
        int Send(const std::vector<u8>& payload);

        /// Sends a message to a specified IPAddress.
        int SendTo(const IPAddress& ipaddr, const u8 *data, unsigned int length, int flags);
        int SendTo(const IPAddress& ipaddr, const std::vector<u8> data);

        int Receive(IPAddress& ipaddr, const u8 *message) const;
        int ReceiveFromAny(const u8 *message);

		int Broadcast(int opt) const;
		void Interrupt() const;
	private:
		int sock{-1};
		sockaddr_in_t self_addr{};
		socklen_t self_addr_len = sizeof(self_addr);
		sockaddr_in_t peer_addr{};
		socklen_t peer_addr_len = sizeof(peer_addr);
		bool connected = false;


    };
}
