#include <Sockets/Endianness.hpp>
#include <bit>
#include <cstdint>


#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#elif __linux__
#include <netinet/in.h>
#endif


namespace Socket
{
    namespace Endianness
    {

        u16 HostToNetworkOrder(u16 host) {
            return htons(host);
        }

        u16 NetworkToHostOrder(u16 network) {
            return ntohs(network);
        }

        u32 HostToNetworkOrder(u32 host) {
            return htonl(host);
        }

        u32 NetworkToHostOrder(u32 network) {
            return ntohl(network);
        }

        u64 HostToNetworkOrder(u64 host) {
            if (IsLittleEndian())
                return ReverseByteOrder(host);
            else
                return host;
        }

        u64 NetworkToHostOrder(u64 network) {
            if (IsLittleEndian())
                return ReverseByteOrder(network);
            else
                return network;
        }

        float HostToNetworkOrder(float host) {
            if (IsLittleEndian())
                return ReverseByteOrder(host);
            else
                return host;
        }

        float NetworkToHostOrder(float network) {
            if (IsLittleEndian())
                return ReverseByteOrder(network);
            else
                return network;
        }

        double HostToNetworkOrder(double host) {
            if (IsLittleEndian())
                return ReverseByteOrder(host);
            else
                return host;
        }

        double NetworkToHostOrder(double network) {
            if (IsLittleEndian())
                return ReverseByteOrder(network);
            else
                return network;
        }
    }
}