#include <Sockets/IPAddress.hpp>
#include <Sockets/Exceptions.hpp>
#include <jtest/jtest.hpp>
#include <jtest/Unit.hpp>

jtest::Unit IPAddrUnit {"IPAddress"};

namespace IPAddressTests
{

    inline void Run()
    {
        IPAddrUnit.RunAll();
    }

    inline void Define()
    {
        using namespace Socket;
        using namespace jtest;

        IPAddrUnit += Test("DefaultCtor", []
        {
            Socket::IPAddress addr;
            check(addr.addr_string() == "0.0.0.0");
            check(addr.port == 0);
        });

        IPAddrUnit += Test("StringCtor", []
        {
            Socket::IPAddress addr("127.0.0.1", 4444);
            check(addr.addr_string() == "127.0.0.1");
            check(addr.port == 4444);
        });

        IPAddrUnit += Test("OctetCtor", []
        {
            Socket::IPAddress addr(255, 0, 255, 0, 42069);
            check(addr.octets[0]==255);
            check(addr.octets[1]==0);
            check(addr.octets[2]==255);
            check(addr.octets[3]==0);
            check(addr.port == 42069);
        });

        IPAddrUnit += Test("CtorWithInvalidAddress", []
        {
            check_death([]{
                                 Socket::IPAddress addr("127.0.0,", 4444);
                         });
            check_death([]{
                                 Socket::IPAddress addr("www.google.com", 4444);
                         });
        });

        IPAddrUnit += Test("Resolve", []
        {
            Socket::IPAddress  addr = Socket::IPAddress::Resolve("www.google.com", 80);

        });

        IPAddrUnit += Test("ResolveWithInvalidUrl", []
        {
            check_death([]{
                                 Socket::IPAddress addr = Socket::IPAddress::Resolve("GOOGLEPLS", 80);
                         });
        });

        IPAddrUnit += Test("operator == ()", []
        {
            Socket::IPAddress a("127.0.0.1", 42069);
            Socket::IPAddress b("127.0.0.1", 42069);
            Socket::IPAddress c("255.255.255.255", 666);

            check(a == b);
            check(a != c);
        });
    }
}






